/*
 * Copyright (c) 2008-2010
 *	Nakata, Maho
 * 	All rights reserved.
 *
 * $Id: dgemm.cpp,v 1.5 2010/08/19 01:29:39 nakatamaho Exp $
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <complex>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define F77_FUNC(name,NAME) name ## _
#include <blas.h>
#define ___DOUBLE_BENCH___
#include <mpack_benchmark.h>

#define TOTALSTEPS 1000

int
main (int argc, char *argv[])
{
  double alpha, beta, dummy;
  double *dummywork;
  double elapsedtime, t1, t2;
  char transa, transb, normtype;
  int N0, M0, K0, STEPN, STEPM, STEPK;
  int lda, ldb, ldc;
  int i, j, m, n, k, ka, kb, p, q;

  // initialization
  N0 = M0 = K0 = 1;
  STEPM = STEPN = STEPK = 1;
  transa = transb = 'n';
  normtype = 'm';
  if (argc != 1) {
    for (i = 1; i < argc; i++) {
      if (strcmp ("-N", argv[i]) == 0) {
	N0 = atoi (argv[++i]);
      }
      else if (strcmp ("-M", argv[i]) == 0) {
	M0 = atoi (argv[++i]);
      }
      else if (strcmp ("-K", argv[i]) == 0) {
	K0 = atoi (argv[++i]);
      }
      else if (strcmp ("-STEPN", argv[i]) == 0) {
	STEPN = atoi (argv[++i]);
      }
      else if (strcmp ("-STEPM", argv[i]) == 0) {
	STEPM = atoi (argv[++i]);
      }
      else if (strcmp ("-STEPK", argv[i]) == 0) {
	STEPK = atoi (argv[++i]);
      }
      else if (strcmp ("-NN", argv[i]) == 0) {
	transa = transb = 'n';
      }
      else if (strcmp ("-TT", argv[i]) == 0) {
	transa = transb = 't';
      }
      else if (strcmp ("-NT", argv[i]) == 0) {
	transa = 'n';
	transb = 't';
      }
      else if (strcmp ("-TN", argv[i]) == 0) {
	transa = 't';
	transb = 'n';
      }
    }
  }

  m = M0;
  n = N0;
  k = K0;
  for (p = 0; p < TOTALSTEPS; p++) {
    if (lsame_f77 (&transa, "n")) {
      ka = k;
      lda = m;
    }
    else {
      ka = m;
      lda = k;
    }
    if (lsame_f77 (&transb, "n")) {
      kb = n;
      ldb = k;
    }
    else {
      kb = k;
      ldb = n;
    }
    ldc = m;

    double *A = new double[lda * ka];
    double *B = new double[ldb * kb];
    double *C = new double[ldc * n];
    double mOne = -1;
    alpha = randomnumber (dummy);
    beta = randomnumber (dummy);
    for (i = 0; i < lda * ka; i++) {
      A[i] = randomnumber (dummy);
    }
    for (i = 0; i < ldb * kb; i++) {
      B[i] = randomnumber (dummy);
    }
    for (i = 0; i < ldc * n; i++) {
      C[i] = randomnumber (dummy);
    }
    t1 = gettime ();
    dgemm_f77 (&transa, &transb, &m, &n, &k, &alpha, A, &lda, B, &ldb, &beta, C, &ldc);
    t2 = gettime ();
    elapsedtime = (t2 - t1);
    printf ("    m     n     k     MFLOPS    transa   transb\n");
    //2mnk+2mn flops are needed
    printf ("%5d %5d %5d %10.3f         %c        %c\n", (int) m, (int) n, (int) k, (2.0 * (double) m * (double) n * (double) k + 2.0 * (double) m * (double) n) / elapsedtime * MFLOPS, transa, transb);
    delete[]C;
    delete[]B;
    delete[]A;
    m = m + STEPM;
    n = n + STEPN;
    k = k + STEPK;
  }
}
